

## Please install Lavaan package if have not.
## install.packages("lavaan")

## Bain package is not on CRAN at the moment
## Please download Bain package from 
## https://informative-hypotheses.sites.uu.nl/software/bain/
## and install it from local file using following lines.
## install.packages(".../Bain_xxx.zip", repos = NULL, type = "win.binary")  ##for Windows
## install.packages(".../Bain_xxx.tgz", repos = NULL, type = "mac.binary")  ##for mac


rm(list=ls())
# Load Lavaan and Bain
library(lavaan)
library(Bain)

# Specify the CFA model
CFA.model <- 'visual  =~ x1 + x2 + x3
              textual =~ x4 + x5 + x6
              speed   =~ x7 + x8 + x9'
fit<-cfa(CFA.model,data=HolzingerSwineford1939,std.lv = TRUE)
n<-nobs(fit)  ##sample size


##Because hypothesis H1 and H2 in equation (43) and (44) are NOT competing, 
##we test them separately.

##For hypothesis H1
##observe estimates and covariance matrix of factor loadings
estimate1<-standardizedSolution(fit)[1:9,4]
Sigma1<-lavInspect(fit, "vcov.std.all")[1:9,1:9]
p1<-length(estimate1)  ##number of parameters

ERr1=NULL  ## no equality constraints
##order constraints
IRr1<-matrix(c(1, -1,  0,  0,  0,  0,  0,  0,  0,  0,
               1,  0, -1,  0,  0,  0,  0,  0,  0,  0,
               0,  0,  0,  1, -1,  0,  0,  0,  0,  0,
               0,  0,  0,  1,  0, -1,  0,  0,  0,  0,
               0,  0,  0,  0,  0,  0,  1, -1,  0,  0,
               0,  0,  0,  0,  0,  0,  1,  0, -1,  0),
             nrow = 6, byrow = TRUE)

##run Bain function
res1<-Bain(estimate = estimate1, Sigma = Sigma1, grouppara = 0, jointpara = p1, n = n, 
          ERr1, IRr1)

##test results
res1$testResult
res1$BFmatrix

##For hypothesis H2
##observe estimates and covariance matrix of correlations
estimate2<-standardizedSolution(fit)[22:24,4]
Sigma2<-lavInspect(fit, "vcov.std.all")[19:21,19:21]
p2<-length(estimate2)  ##number of parameters

ERr2<-NULL ## no equality constraints
##order constraints
IRr2<-matrix(c(1, -1,  0, 0,
               1,  0, -1, 0), nrow = 2, byrow = TRUE)

##run Bain function
res2<-Bain(estimate = estimate2, Sigma = Sigma2, grouppara = 0, jointpara = p2, n = n, 
           ERr2, IRr2)

##test results
res2$testResult
res2$BFmatrix

