
!************************************************************************************************************************
!                                                                                                                       !
!  PROGRAM: BIG                                                                                                         !
!  This program is used to compute the Bayes factor of inquality constrained hypotheses.                                !
!  I provide the code such that everyone can verify for themselves.                                                     !
!  The prior distribution used in this program is N(0,w*I), where I is a identity matrix and w is approaching infinite  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                                                       !
!************************************************************************************************************************

  include 'link_f90_static.h'   ! Compiler V9
! include 'link_fnl_static.h'   ! Compiler V10 		

program BIG

    USE IMSL_LIBRARIES
    
    ! Declare variables
    implicit none
    ! Integers
    integer                             :: i, ii, j, k, s, cout2, time, ISEED, ss, SSS
    integer								:: ios, numR, length, N, discard

    ! Reals/doubles
    real                                :: p, gibbsa, GG, LO, UP, temp, MSE,  totalc
    real, dimension(:), allocatable     :: initial, parmeans, means, x, y, cc, LOWER, UPPER
    real, dimension(:), allocatable     :: large, small
    real, dimension(:,:), allocatable   :: Rr, IE, priorcov, priorinvcov, priorC
    character*256                       :: line

! Open two .txt files: one is for input and the other one is for output.
Open(unit = 10, file = 'Input.txt', status ='OLD', iostat=ios)
Open(unit = 20, file = 'Output.txt', status = 'replace')

read(10,*)
read(10,*) N
! Read the number of parameters and constraints
read(10,*)
read(10,*) length, numR

! Allocate variables, where "parmeans" denotes estimates of parameters and "cov" denotes the covariance matrix of parameters.
! Note that Rtheta>r will be transfered into [R|r]theta>0, where [R|r] is an augmented matrix, denoted by "Rr". 
! Therefore, the theta vector "parmeans" is a length+1 vector with the last element being -1. 
allocate (Rr(numR, length+1))

! Read the restriction matrix (R-r or [R|r])
read(10,*)
do i = 1, numR
   read(10,*) Rr(i,1:(length+1)) ! Note that here Rr=[R|r] 	
end do

close(10)

! Provide a seed for random number generation
ISEED=12345
call RNSET(ISEED)! initialize a random seed for use in the IMSL random number generators


    allocate (y(length+1),means(length+1),LOWER(length+1),UPPER(length+1))
    allocate (cc(numR),large(numR), small(numR))
    ! matrixes
    allocate (IE(length,length))
    allocate (priorcov(length,length), priorinvcov(length,length), priorC(length,length))

    IE=0
    ! "B" is a diagonal matrix of "invcov"
    do i=1,length
       IE(i,i)=1
    end do
     
    priorcov=10000*IE  
    priorinvcov=IE/10000
    priorC=0


    MSE=0
    SSS=1000
    
    
    do ss=1,SSS
    print '(I3)', ss
    do ii=1,numR ! Decompose the Bayes factor, "ii" denotes each line of the restriction matrix (R-r). 
                 ! We compute the complexity and fit in each line of (R-r).
       discard=100   
       
  888  cout2=0 ! "cout2" is used for counting the number of iterations that are in agreement with "ii"th constraints
       y=0 ! initial value

       do time=1,N+discard ! "time" denotes the iteration index
          means=0 
          do i=1,length
          
          ! "means" is the mean of conditional prior normal distribution of theta_(i), 
             means(i)=sum(priorC(i,1:length)*y(1:length))-priorC(i,i)*y(i)
             
          ! Each sample should be in line with the first "ii-1" constraints in (R-r)   
             if(ii > 1) then ! except the first line "ii=1" of R, because the sample of "ii=1" is not constrained.
                do k=1,(ii-1) ! first "ii-1" constraints
                
                   if (Rr(ii-k,i)>0) then
                   ! The parameter should be larger than (define a lower bound of parameter "i")
                      large(k)=(Rr(ii-k,i)*y(i)-sum(Rr(ii-k,1:(length+1))*y))/Rr(ii-k,i)
                   else ! if no lower constaint
                      large(k)=-10**20
                   end if
                  
                   if (Rr(ii-k,i)<0) then
                   ! The parameter should be samller than (define an upper bound of parameter "i") 
                      small(k)=(Rr(ii-k,i)*y(i)-sum(Rr(ii-k,1:(length+1))*y))/Rr(ii-k,i)
                   else ! if no upper constaint
                      small(k)=10**20
                   end if
                end do
                
                ! The last "numR-ii+1" constraints are not used
                do s=ii,numR ! no constraint
                   large(s)=-10**20
                   small(s)=10**20
                end do
                
                ! Compute the lower and upper bound, the parameter should be sampled in that interval
                ! Standardize the lower and upper bound. Note that the prior mean is "priormeans" and the variance is "1/priorinvcov(i,i)".
                LO=MAXVAL(large)/sqrt(1/priorinvcov(i,i))
                UP=MINVAL(small)/sqrt(1/priorinvcov(i,i))
                GG=RNUNF() ! This function generates a pseudorandom number from a uniform (0,1) distribution
                
                ! "ANORDF" is a function that evaluates the standard normal distribution function.
                ! This function return a probability that a normal random variable takes a value less than or equal to (input)
                p=ANORDF(LO)+GG*(ANORDF(UP)-ANORDF(LO))
                
                ! Avoid p=1 and p=0, which are not accapted by "ANORIN" function
                if (p>0.999999)then
                p=0.999999
                end if
                if (p<0.000001)then
                p=0.000001
                end if
                
                ! "ANORIN" is a function that evaluates the inverse of the strandard normal distribution
                gibbsa=ANORIN(p)
                y(i)=gibbsa*sqrt(1/priorinvcov(i,i))
             else !if the first line "ii=1" of R
                GG=RNNOF()
		        y(i)=GG*sqrt(1/priorinvcov(i,i))
             end if
          end do
          
          ! Count the number of sample that are in line with the "ii" constraints 
          ! Note that time>discard is to discard the first 100 burn-in sample    
          if(sum(Rr(ii,1:(length+1))*y)>0 .AND. time>discard) then
             cout2=cout2+1
          end if
       end do
       
       ! Compute the complexity in "ii" constraints
       cc(ii)=real(cout2)/real(N)
    end do
    
    totalc=1
    do i=1,numR 
    totalc=totalc*cc(i)
    end do
    
    if (numR==2)then
       MSE=MSE+1/real(SSS)*((totalc-1/real(6))*real(6))**2
    end if
    if (numR==3)then
       MSE=MSE+1/real(SSS)*((totalc-1/real(24))*real(24))**2
    end if
    if (numR==4)then
       MSE=MSE+1/real(SSS)*((totalc-1/real(120))*real(120))**2
    end if
     
    end do
    
write(20,'(A,i1)') '  MSER for J=', length
write(20,*) MSE

close(20)     
end program BIG
