
!************************************************************************************************************************
!                                                                                                                       !
!  PROGRAM: BIG                                                                                                         !
!  This program is used to compute the Bayes factor of inquality constrained hypotheses.                                !
!  I provide the code such that everyone can verify for themselves.                                                     !
!  The prior distribution used in this program is N(0,w*I), where I is a identity matrix and w is approaching infinite  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                                                       !
!************************************************************************************************************************

  include 'link_f90_static.h'   ! Compiler V9
! include 'link_fnl_static.h'   ! Compiler V10 		

program BIG

    USE IMSL_LIBRARIES
    
    ! Declare variables
    implicit none
    ! Integers
    integer                             :: i, ii, j, k, time, ISEED, s, SS
    integer								:: ios, numR, length, N, discard

    ! Reals/doubles
    real                                :: p, GG, LO, temp, pp, MSE,totalc
    real, dimension(:), allocatable     :: parmeans, means, y, Rtheta, cc
    real, dimension(:,:), allocatable   :: Rr, priorC, IE, invRmatrixprior,invDiagprior
    real, dimension(:,:), allocatable   :: Rmatrixprior
    character*256                       :: line

! Open two .txt files: one is for input and the other one is for output.
Open(unit = 10, file = 'Input.txt', status ='OLD', iostat=ios)
Open(unit = 20, file = 'Output.txt', status = 'replace')
read(10,*)
read(10,*) N
! Read the number of parameters and constraints
read(10,*)
read(10,*) length, numR

! Allocate variables, where "parmeans" denotes estimates of parameters and "cov" denotes the covariance matrix of parameters.
! Note that Rtheta>r will be transfered into [R|r]theta>0, where [R|r] is an augmented matrix, denoted by "Rr". 
! Therefore, the theta vector "parmeans" is a length+1 vector with the last element being -1. 
allocate (Rr(numR, length+1))
 
! Read the restriction matrix (R-r or [R|r])
read(10,*)
do i = 1, numR
   read(10,*) Rr(i,1:(length+1)) ! Note that here Rr=[R|r] 	
end do

close(10)

! Provide a seed for random number generation
ISEED=12345
call RNSET(ISEED)! initialize a random seed for use in the IMSL random number generators

! U is the number of iteration for computing MC error
    ! Allocate variable vectors and matrixes needed in the program
    ! vectors
    allocate (y(numR),means(numR))
    allocate (cc(numR),Rtheta(numR))
    ! matrixes
    allocate (IE(numR,numR),invDiagprior(numR,numR),Rmatrixprior(numR,numR))
    allocate (invRmatrixprior(numR,numR),priorC(numR,numR))

    call MXYTF(Rr(1:numR,1:length),Rr(1:numR,1:length),Rmatrixprior)
    

    ! Calculate the matrix used in the conditional posterior distribution, i.e., C=I-[diag(cov^(-1))]^(-1)cov^(-1)
    IE=0
    Rmatrixprior=10000*Rmatrixprior
    call LINRG(Rmatrixprior,invRmatrixprior)
    ! "invDiag" is the inverse matrix of the diagonal matrix of "invDiag": invRmatrix=invRmatrix^(-1)
    do i=1,numR
       invDiagprior(i,i)=(invRmatrixprior(i,i))**(-1)
       IE(i,i)=1
    end do
    
    ! compute C=I-[diag(Rmatrix^(-1))]^(-1)Rmatrix^(-1)
    do i=1,numR
       do j=1,numR
          priorC(i,j)=IE(i,j)-invDiagprior(i,i)*invRmatrixprior(i,j)
       end do
    end do  
    
    MSE=0
    SS=1000
    do s=1,SS 
    do ii=1,numR 
       discard=100   
       pp=0  
   888 y=0 ! initial value
       do time=1,N+discard ! "time" denotes the iteration index
          means=0 
          do i=1,numR
          
          ! Each sample should be in line with the first "ii-1" constraints in (R-r)   
            ! "means" is the mean of the conditional posterior normal distribution of theta_(i)
             means(i)=sum(priorC(i,1:numR)*y(1:numR)-priorC(i,i)*y(i))
             
             ! Each sample should be in line with the first "ii-1" constraints in (R-r)
             if(i<ii) then

                ! Compute the lower and upper bound, the parameter should be sampled in that interval
                ! Standardize the lower and upper bound, because the function "ANORDF" requires the standard normal distribution.
                LO=(0-means(i))/sqrt(1/invRmatrixprior(i,i)) 
                GG=RNUNF() ! This function generates a pseudorandom number from a uniform (0,1) distribution
                
                ! "ANORDF" is a function that evaluates the standard normal distribution function.
                ! This function return a probability that a normal random variable takes a value less than or equal to (input)
                p=ANORDF(LO)+GG*(1-ANORDF(LO))
                
                ! Avoid p=1 and p=0, which are not accapted by "ANORIN" function
                if (p>0.999999)then
                p=0.999999
                end if
                if (p<0.000001)then
                p=0.000001
                end if
                
                ! "ANORIN" is a function that evaluates the inverse of the strandard normal distribution function.
                temp=ANORIN(p)
                ! Compute the sample with a mean of "means(i)" and a variance of "1/invcov(i,i)"
                y(i)=temp*sqrt(1/invRmatrixprior(i,i))+means(i) 
             else 
                GG=RNNOF()
		        y(i)=GG*sqrt(1/invRmatrixprior(i,i))+means(i)
             end if   
          end do
          
          if (time>discard)then
              temp=means(ii)/sqrt(1/invRmatrixprior(ii,ii))

              pp=pp+ANORDF(temp)
          end if 
       end do
       cc(ii)=pp/real(N)
    end do
    
    totalc=1
    do i=1,numR 
    totalc=totalc*cc(i)
    end do
    
    if (numR==2)then
       MSE=MSE+1/real(SS)*((totalc-1/real(6))*real(6))**2
    end if
    if (numR==3)then
       MSE=MSE+1/real(SS)*((totalc-1/real(24))*real(24))**2
    end if
    if (numR==4)then
       MSE=MSE+1/real(SS)*((totalc-1/real(120))*real(120))**2
    end if
    
    end do

write(20,'(A,i1)') '  MSER for J=', length
write(20,*) MSE

close(20)     
end program BIG
